@@ -2,7 +2,7 @@ package Lingua::TokenParse;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.10.2';
+$VERSION = '0.12.1';
 
 # NOTE: The {{{ and }}} things are "editor code fold markers".  They
 # are merely a convenience for people who don't care to scroll through
@@ -17,7 +17,7 @@ sub new {  # {{{
         # The word to parse!
         word         => $args{word} || undef,
         # We need to use the length of our word in a few methods.
-        _word_length => exists $args{word} ? length ($args{word}) : undef,
+        _word_length => exists $args{word} ? length ($args{word}) : 0,
         # Known tokens.
         lexicon      => $args{lexicon} || {},
         # All word parts.
@@ -30,10 +30,14 @@ sub new {  # {{{
         definitions  => {},
         # Fragment definition separator.
         separator    => $args{separator} || ' + ',
+        # Known-but-not-defined definition output string.
+        not_defined  => $args{not_defined} || '.',
+        # Unknown definition output string.
+        unknown      => $args{unknown} || '?',
         # Known trimming regexp rules.
         rules        => [],
         # Globals used by the build_combinations method (and
-        # initialized by the _reset_parse() method).
+        # initialized by the _reset_parse method).
         _new  => [],
         _prev => 0,
     };
@@ -91,6 +95,18 @@ sub separator {  # {{{
     return $self->{separator};
 }  # }}}
 
+sub not_defined {  # {{{
+    my $self = shift;
+    $self->{not_defined} = shift if @_;
+    return $self->{not_defined};
+}  # }}}
+
+sub unknown {  # {{{
+    my $self = shift;
+    $self->{unknown} = shift if @_;
+    return $self->{unknown};
+}  # }}}
+
 sub rules {  # {{{
     my $self = shift;
     $self->{rules} = shift if @_;
@@ -172,10 +188,10 @@ sub build_knowns {  # {{{
 
         for (@chunks) {
             # Handle hyphens in lexicon entries.
-            ($_, my $flag) = _hyphenate($_, $self->lexicon, 0);
+            ($_, my $combo_seen) = _hyphenate($_, $self->lexicon, 0);
 
             # Sum the combination familiarity value.
-            if ($flag) {
+            if ($combo_seen) {
                 $frag_sum++;
                 $char_sum += length;
             }
@@ -183,7 +199,7 @@ sub build_knowns {  # {{{
 
         $combo = join '.', @chunks;
 
-        # Save this combination with it's familiarity ratio.
+        # Save this combination with its familiarity ratio.
         $self->knowns->{$combo} = [
             $frag_sum / @chunks,
             $char_sum / $self->{_word_length}
@@ -233,8 +249,8 @@ sub trim_knowns {  # {{{
         push @seen, scalar _hyphenate($unknown, $self->lexicon)
             if $unknown;
 
-        # Assign the score of the trimmed combination from the "all
-        # combinations list.
+        # Add the combo to the trimmed combinations list and assign
+        # the score from the "all possible combinations" list.
         $combo = join '.', @seen;
         $trimmed{$combo} = $self->knowns->{$combo};
     }
@@ -244,14 +260,13 @@ sub trim_knowns {  # {{{
     for my $combo (sort keys %trimmed) {
         # Initialize the "bogus combination flag" for use with the
         # index() function.
-        my $flag = -1;
+        my $seen_position = -1;
 
-        # Inspect each combination chunk.
+        # Inspect each unknown chunk.
         for my $chunk (split /\./, $combo) {
-            next if $self->definitions->{$chunk};
+            next if defined $self->definitions->{$chunk};
 
-            # Loops through the defined fragments.
-            # Does the unknown chunk contain a defined lexicon entry?
+            # Do we contain a defined lexicon entry?
             for my $entry (
                 grep { defined $self->definitions->{$_} }
                     sort keys %{ $self->definitions }
@@ -259,58 +274,60 @@ sub trim_knowns {  # {{{
                 # Strip off the stem-hyphen, if it exists.
                 $entry =~ s/-//;
 
-                # Flag the combination as bogus, if it has an
-                # unknown bit that contains a known fragment.
-                $flag = index $chunk, $entry;
+                # Set the seen_position flag if we contain a known
+                # fragment.
+                $seen_position = index $chunk, $entry;
 
                 # Bail out, if we found a bogus combination.
-                last if $flag >= 0;
+                last if $seen_position >= 0;
             }
 
             # Bail out, if we found a bogus combination.
-            last if $flag >= 0;
+            last if $seen_position >= 0;
         }
 
         # Remove combinations that were flagged as being bogus.
-        delete $trimmed{$combo} if $flag >= 0;
+        delete $trimmed{$combo} if $seen_position >= 0;
     }
 
     # Execute user provided regular expressions.
     for my $combo (sort keys %trimmed) {
-        my $flag = 0;
+        my $matched = 0;
 
-        # Flag matching rules.
+        # Flag matching rules as bogus.
         for my $rule (@{ $self->rules }) {
             if ($combo =~ /$rule/) {
-                $flag++;
+                $matched++;
                 last;
             }
         }
 
         # Remove this combination if it was flagged as being bogus.
-        delete $trimmed{$combo} if $flag;
+        delete $trimmed{$combo} if $matched;
     }
 
     # Set the knowns list to the new trimmed list.
     $self->knowns(\%trimmed);
 }  # }}}
 
+# Update the given string with it's actual lexicon value and increment
+# the seen flag.
 sub _hyphenate {  # {{{
-    my ($string, $lex, $flag) = @_;
+    my ($string, $lexicon, $combo_seen) = @_;
 
-    if (exists $lex->{$string}) {
-        $flag++ if defined $flag;
+    if (exists $lexicon->{$string}) {
+        $combo_seen++ if defined $combo_seen;
     }
-    elsif (exists $lex->{"-$string"}) {
-        $flag++ if defined $flag;
+    elsif (exists $lexicon->{"-$string"}) {
+        $combo_seen++ if defined $combo_seen;
         $string = "-$string";
     }
-    elsif (exists $lex->{"$string-"}) {
-        $flag++ if defined $flag;
+    elsif (exists $lexicon->{"$string-"}) {
+        $combo_seen++ if defined $combo_seen;
         $string = "$string-";
     }
 
-    return wantarray ? ($string, $flag) : $string;
+    return wantarray ? ($string, $combo_seen) : $string;
 }  # }}}
 
 sub output_knowns {  # {{{
@@ -319,8 +336,8 @@ sub output_knowns {  # {{{
 
     my $header = <<HEADER;
 Combination [fragment familiarity, character familiarity]
-Fragment definitions (with the defined fragment separator and a ?
-character for unknowns).
+Fragment definitions (with the fragment separator, a period for known
+but not defined, and a question mark for unknowns).
 
 HEADER
 
@@ -333,9 +350,11 @@ HEADER
         my @definition;
         for my $chunk (split /\./) {
             push @definition,
-                $self->definitions->{$chunk}
+                defined $self->definitions->{$chunk}
                     ? $self->definitions->{$chunk}
-                    : '?';
+                        ? $self->definitions->{$chunk}
+                        : $self->not_defined
+                    : $self->unknown;
         }
 
         push @out, sprintf qq/%s [%s]\n%s/,
@@ -362,9 +381,8 @@ Lingua::TokenParse - Parse a word into scored, fragment combinations
   my %lexicon;
   @lexicon{qw(art ion ti)} = qw(foo bar baz);
   my $obj = Lingua::TokenParse->new(
-      word      => $word,
-      lexicon   => \%lexicon,
-      separator => ' ~ ',
+      word    => $word,
+      lexicon => \%lexicon,
   );
   print scalar $obj->output_knowns;
 
@@ -372,12 +390,12 @@ Lingua::TokenParse - Parse a word into scored, fragment combinations
   $obj->word('metaphysical');
   $obj->lexicon({
       'meta-' => 'more comprehensive',
-      'ta'    => 'foo',
+      'ta'    => '',
       'phys'  => 'natural science, singular',
       '-ic'   => 'being, containing',
       '-al'   => 'relating to, characterized by',
   });
-  $obj->rules([ qr/^me\./ ]);  # Yank combos that start with "me."
+  $obj->rules([ qr/^me\./ ]);  # Remove combos that start with "me."
   $obj->parse;
   my @knowns = $obj->output_knowns;
 
@@ -395,13 +413,15 @@ partition this word into combinations of these (possibly overlapping)
 parts.  Each of these combinations can be given a score, which 
 represents a measure of familiarity.
 
-Currently, this familiarity measure is a simple ratio of known to 
+This familiarity measure is a set of simple ratios of known to 
 unknown parts.
 
 Note that the lexicon must have definitions for each entry, in order 
-to have the current trim_knowns() method do the right thing.
+to have the current trim_knowns method do the right thing.  The 
+definition can be an empty string (i.e. '').  If the definition is 
+undefined, the fragment is considered an unknown.
 
-* Check out the sample code in the distribution's eg/ directory for 
+Please see the sample code in the distributions eg/ directory for 
 examples of how this module can be used.
 
 =head1 METHODS
@@ -409,8 +429,11 @@ examples of how this module can be used.
 =head2 new()
 
   $obj = Lingua::TokenParse->new(
-      word    => $word,
-      lexicon => \%lexicon,
+      word        => $word,
+      lexicon     => \%lexicon,
+      separator   => $separator,
+      not_defined => $not_defined,
+      unknown     => $unknown,
   );
 
 Return a new Lingua::TokenParse object.
@@ -418,6 +441,26 @@ Return a new Lingua::TokenParse object.
 This method will automatically call the partition methods (detailed 
 below) if a word and lexicon are provided.
 
+The word can be any string, however, you will want to make sure that 
+it does not include the same characters you use for the separator,
+not_defined and unknown strings (described below).
+
+The lexicon must be a hash reference with word fragments as keys and
+definitions their respecive values.  Definitions must be defined in 
+order for the trim_knowns method work properly.
+
+The separator is the string used to separate fragment definitions in
+the output_knowns method.  The default is a plus symbol surrounded 
+by single spaces (' + ').
+
+The not_defined argument is the the string used by the output_knowns 
+method to indicate a known fragment that has no 
+definition.  The default is a period (.).
+
+The unknown argument is the the string used by the output_knowns 
+method to indicate an unknown fragment.  The default is the question
+mark (?).
+
 =head2 parse()
 
   $obj->parse();
@@ -432,7 +475,7 @@ optionally, a new lexicon.
 
   $obj->build_parts();
 
-Construct an array of the word partitions, accessed via the parts() 
+Construct an array of the word partitions, accessed via the parts
 method.
 
 =head2 build_combinations()
@@ -440,13 +483,13 @@ method.
   $obj->build_combinations();
 
 Recursively compute the array of all possible word part combinations,
-accessed via the combinations() method.
+accessed via the combinations method.
 
 =head2 build_knowns()
 
   $obj->build_knowns();
 
-Compute the familiar word part combinations, accessed via the knowns()
+Compute the familiar word part combinations, accessed via the knowns
 method.
 
 This method handles word parts containing prefix and suffix hyphens,
@@ -460,7 +503,7 @@ combinations).
 
 Construct a hash of the definitions of the word parts in each 
 combination in the keys of the knowns hash.  This hash is accessed
-via the definitions() method.
+via the definitions method.
 
 =head2 trim_knowns()
 
@@ -496,7 +539,7 @@ Here is the format of the output:
 These accessors both get and set their respective values.  Note 
 that, if you set the word, lexicon or rules after construction, you 
 must manually initialize the parse lists and run the partition 
-methods (via the parse() method).
+methods (via the parse method).
 
 Also, note that it is useless to set the parts, combinations and 
 knowns lists, since they are computed by the partition methods.
@@ -507,12 +550,17 @@ knowns lists, since they are computed by the partition methods.
 
 The actual word to partition.
 
+Ths word can be any string, however, you will want to make sure that 
+it does not include the same characters you use for the separator,
+not_defined and unknown strings.
+
 =head2 lexicon()
 
   $lexicon = $obj->lexicon(\%lexicon);
 
-The hash reference of word parts (keys) with their (optional) 
-definitions (values).
+The lexicon is a hash reference with word fragments as keys and
+definitions their respecive values.  Definitions must be defined in 
+order for the trim_knowns method to work properly.
 
 =head2 parts()
 
@@ -521,7 +569,7 @@ definitions (values).
 The array reference of word partitions.
 
 Note that this method is only useful for fetching, since the parts 
-are computed by the build_parts() method.
+are computed by the build_parts method.
 
 =head2 combinations()
 
@@ -530,7 +578,7 @@ are computed by the build_parts() method.
 The array reference of all possible word part combinations.
 
 Note that this method is only useful for fetching, since the 
-combinations are computed by the build_combinations() method.
+combinations are computed by the build_combinations method.
 
 =head2 knowns()
 
@@ -541,7 +589,7 @@ familiarity scores (values).  Note that only the non-zero scored
 combinations are kept.
 
 Note that this method is only useful for fetching, since the knowns
-are computed by the build_knowns() method.
+are computed by the build_knowns method.
 
 =head2 definitions()
 
@@ -551,15 +599,6 @@ The hash reference of the definitions provided for each fragment of
 the combinations in the knowns hash.  Note that the unknown 
 fragments are defined as an empty string.
 
-=head2 separator()
-
-  $separator = $obj->separator($separator);
-
-The character (or characters) separating the fragment definitions that
-are produced by the output_knowns() method.
-
-The default is ' + ' (a plus symbol surrounded by single spaces).
-
 =head2 rules()
 
   $rules = $obj->rules(\@rules);
@@ -569,7 +608,31 @@ the list of known combinations.  If a match is successful, the entry
 is removed from the list.
 
 To reiterate, this is a negative, pruning device, that is used in the
-trim_knowns() method.
+trim_knowns method.
+
+=head2 separator()
+
+  $separator = $obj->separator($separator);
+
+The separator is the string used to separate fragment definitions in
+the output_knowns method.  The default is a plus symbol surrounded 
+by single spaces (' + ').
+
+=head2 not_defined()
+
+  $not_defined = $obj->not_defined($not_defined);
+
+The not_defined argument is the the string used by the output_knowns 
+method to indicate a known fragment that has no 
+definition.  The default is a period (.).
+
+=head2 unknown()
+
+  $unknown = $obj->unknown($unknown);
+
+The unknown argument is the the string used by the output_knowns 
+method to indicate an unknown fragment.  The default is the question
+mark (?).
 
 =head1 DEPENDENCIES
 
@@ -584,8 +647,6 @@ needed improvements.
 
 =head1 TO DO
 
-Add user defined, known combination rule trimming callbacks.
-
 Compute the time required for a given parse.
 
 Synthesize a term list based on word part (thesaurus) definitions.
